@@ -294,24 +294,24 @@ cpl_parameterlist * hdrl_rect_region_parameter_create_parlist(
     hdrl_setup_vparameter(parlist, prefix, ".", name_prefix,
                          "llx", base_context,
                          "Lower left x pos. (FITS) defining the region",
-                         CPL_TYPE_INT, hdrl_rect_region_get_llx(defaults));
+                         CPL_TYPE_INT, (int)hdrl_rect_region_get_llx(defaults));
     /* --prefix.lly */
     hdrl_setup_vparameter(parlist, prefix, ".", name_prefix,
                          "lly", base_context,
                          "Lower left y pos. (FITS) defining the region",
-                         CPL_TYPE_INT, hdrl_rect_region_get_lly(defaults));
+                         CPL_TYPE_INT, (int)hdrl_rect_region_get_lly(defaults));
 
     /* --prefix.urx */
     hdrl_setup_vparameter(parlist, prefix, ".", name_prefix,
                          "urx", base_context,
                          "Upper right x pos. (FITS) defining the region",
-                         CPL_TYPE_INT, hdrl_rect_region_get_urx(defaults));
+                         CPL_TYPE_INT, (int)hdrl_rect_region_get_urx(defaults));
 
     /* --prefix.ury */
     hdrl_setup_vparameter(parlist, prefix, ".", name_prefix,
                          "ury", base_context,
                          "Upper right y pos. (FITS) defining the region",
-                         CPL_TYPE_INT, hdrl_rect_region_get_ury(defaults));
+                         CPL_TYPE_INT, (int)hdrl_rect_region_get_ury(defaults));
 
     if (cpl_error_get_code()) {
         cpl_parameterlist_delete(parlist);
@@ -800,6 +800,138 @@ cpl_vector * hdrl_image_to_vector(
     return vec_source;
 }
 
+typedef struct {
+    size_t available; /* number of cached pointers */
+    size_t nspace;    /* max cache pointers */
+    void ** ptrs;
+} cache_bucket;
+
+struct hdrl_vector_cache_ {
+    cpl_size nbuckets;
+    cache_bucket buckets[];
+};
+
+/* ---------------------------------------------------------------------------*/
+/**
+ * @brief create a cpl_vector cache
+ *
+ * @param max_cached_size  maximum size of vectors that will be cached
+ * @param ncached_entries  number of vectors of each size that can be cached
+ *
+ * Allows storing allocated vectors in a temporary structure for fast retrieval later.
+ * Useful when one needs to allocate and delete many small vectors with high
+ * frequency, this avoids two calls to malloc that would cause high overheads
+ * in this case.
+ *
+ * Vectors can be retrieved from the cache with hdrl_vector_new_from_cache()
+ * and returned to the cache with hdrl_vector_delete_to_cache().
+ *
+ * The cache can store max_cached_size * ncached_entries vectors.
+ *
+ * May return NULL which is an no-opt cache and is a valid value for functions
+ * working on it.
+ *
+ * @note the cache has no global state but it is not thread-safe
+ *
+ * @return cache structure that must be deleted with hdrl_vector_cache_delete()
+ */
+/* ---------------------------------------------------------------------------*/
+hdrl_vector_cache * hdrl_vector_cache_new(cpl_size max_cached_size,
+                                          cpl_size ncached_entries)
+{
+    /* for largish vectors a cache is not worthwhile, save the memory */
+    if (max_cached_size > 50)
+        return NULL;
+    hdrl_vector_cache * c = cpl_malloc(sizeof(hdrl_vector_cache) +
+                               sizeof(cache_bucket) * (max_cached_size + 1));
+    c->nbuckets = max_cached_size + 1;
+    for (cpl_size i = 0; i < c->nbuckets; i++) {
+        c->buckets[i].available = 0;
+        c->buckets[i].nspace = ncached_entries;
+        c->buckets[i].ptrs = cpl_calloc(sizeof(cpl_vector*), ncached_entries);
+    }
+    return c;
+}
+
+/* ---------------------------------------------------------------------------*/
+/**
+ * @brief Delete cpl_vector cache and all its entries
+ *
+ * @param cache cache to delete
+ */
+/* ---------------------------------------------------------------------------*/
+void hdrl_vector_cache_delete(hdrl_vector_cache * cache)
+{
+    if (!cache)
+        return;
+    for (cpl_size i = 0; i < cache->nbuckets; i++) {
+        for (size_t j = 0; j < cache->buckets[i].available; j++) {
+            cpl_vector_delete(cache->buckets[i].ptrs[j]);
+        }
+        cpl_free(cache->buckets[i].ptrs);
+    }
+    cpl_free(cache);
+}
+
+/* ---------------------------------------------------------------------------*/
+/**
+ * @brief Allocate cpl_vector
+ *
+ * @param cache pointer vector cache structure, may be NULL
+ * @param sz    size of vector to allocate
+ *
+ * returns a cpl_vector as cpl_vector_new() would but if a matching vector is in
+ * the cache it returns that instead of allocating a new one.
+ * The returned vector is a normal cpl_vector with no restrictions on usage.
+ *
+ * @see cpl_vector_new()
+ *
+ * @return  cpl_vector that must be deleted with cpl_vector_delete or
+ *          hdrl_cplvector_delete_to_cache()
+ */
+/* ---------------------------------------------------------------------------*/
+cpl_vector * hdrl_cplvector_new_from_cache(hdrl_vector_cache * cache, cpl_size sz)
+{
+    if (!cache) {
+        return cpl_vector_new(sz);
+    }
+    if (sz < cache->nbuckets) {
+        if (cache->buckets[sz].available > 0) {
+            return cache->buckets[sz].ptrs[--(cache->buckets[sz].available)];
+        }
+    }
+    return cpl_vector_new(sz);
+}
+
+/* ---------------------------------------------------------------------------*/
+/**
+ * @brief Return cpl_vector to a cache
+ *
+ * @param cache  cache to place the vector in, may be NULL
+ * @param v      vector to be deleted or cached
+ *
+ * Returns a vector to the cache for later reuse, if the cache is full it
+ * will be deleted.
+ */
+/* ---------------------------------------------------------------------------*/
+void hdrl_cplvector_delete_to_cache(hdrl_vector_cache * cache, cpl_vector * v)
+{
+    if (!v) {
+        return;
+    }
+    if (!cache) {
+        return cpl_vector_delete(v);
+    }
+    cpl_size sz = cpl_vector_get_size(v);
+    if (sz < cache->nbuckets) {
+        if (cache->buckets[sz].available < cache->buckets[sz].nspace) {
+            cache->buckets[sz].ptrs[(cache->buckets[sz].available)++] = v;
+            return;
+        }
+    }
+    cpl_vector_delete(v);
+}
+
 
 /* internal if imgdatabuf and maskbuf are not NULL they are assumed to hold the
  * data buffers of the image in the list in order to save on cpl_image_get
@@ -809,14 +941,14 @@ static cpl_vector * imagelist_to_vector(const cpl_imagelist * list,
                                         const cpl_size x,
                                         const cpl_size y,
                                         const double ** imgdatabuf,
-                                        const cpl_binary ** maskbuf)
+                                        const cpl_binary ** maskbuf, hdrl_vector_cache * cache)
 {
     const long nz  = list ? cpl_imagelist_get_size(list) : -1;
-    cpl_vector * vec = NULL;
     double * restrict ddata = NULL;
     unsigned long j = 0;
 
-    ddata = cpl_malloc(nz * sizeof(ddata[0]));
+    cpl_vector * vec = hdrl_cplvector_new_from_cache(cache, nz);
+    ddata = cpl_vector_get_data(vec);
 
     if (imgdatabuf && maskbuf) {
         for (long k = 0; k < nz; k++) {
@@ -844,10 +976,13 @@ static cpl_vector * imagelist_to_vector(const cpl_imagelist * list,
     }
 
     if (j > 0) {
-        vec = cpl_vector_wrap(j, ddata);
+        if ((cpl_size)j != nz) {
+            cpl_vector_set_size(vec, j);
+        }
     }
     else {
-        cpl_free(ddata);
+        hdrl_cplvector_delete_to_cache(cache, vec);
+        vec = NULL;
     }
 
     return vec;
@@ -883,7 +1018,7 @@ cpl_vector * hdrl_imagelist_to_vector(const cpl_imagelist * list,
         cpl_ensure(x <= nx, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
         cpl_ensure(y <= ny, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
     }
-    return imagelist_to_vector(list, nx, x, y, NULL, NULL);
+    return imagelist_to_vector(list, nx, x, y, NULL, NULL, NULL);
 }
 
 
@@ -894,6 +1029,8 @@ cpl_vector * hdrl_imagelist_to_vector(const cpl_imagelist * list,
   @param    list     imagelist to compress
   @param    y        y coordinate to compress over (FITS convention)
   @param    out      output array of vector pointers of size nx
+  @param    cache    vector cache to avoid unnecessary memory allocations
+                     may be NULL
   @return  cpl_error_code
   @note    vector can't have size 0 so NULL is returned if all pixels are bad
 
@@ -903,7 +1040,8 @@ cpl_vector * hdrl_imagelist_to_vector(const cpl_imagelist * list,
 /*----------------------------------------------------------------------------*/
 cpl_error_code hdrl_imagelist_to_vector_row(const cpl_imagelist * list,
                                             const cpl_size y,
-                                            cpl_vector ** out)
+                                            cpl_vector ** out,
+                                            hdrl_vector_cache * cache)
 {
     long nx;
     int isdouble;
@@ -936,10 +1074,10 @@ cpl_error_code hdrl_imagelist_to_vector_row(const cpl_imagelist * list,
     for (long x = 0; x < nx; x++) {
         if (isdouble) {
             out[x] = imagelist_to_vector(list, nx, x + 1, y,
-                                         imgdatabuf, maskbuf);
+                                         imgdatabuf, maskbuf, cache);
         }
         else {
-            out[x] = imagelist_to_vector(list, nx, x + 1, y, NULL, NULL);
+            out[x] = imagelist_to_vector(list, nx, x + 1, y, NULL, NULL, cache);
         }
     }
     return cpl_error_get_code();
@@ -1203,7 +1341,7 @@ image_const_row_view_create(const cpl_image * img,
     if (omask) {
         cpl_mask * mask = cpl_mask_wrap(nx, nny,
                         (cpl_binary*)cpl_mask_get_data_const(omask) + offset);
-        cpl_mask_delete(cpl_image_set_bpm(wimg, mask));
+        cpl_mask_delete(hcpl_image_set_bpm(wimg, mask));
     }
 
     return wimg;
@@ -1255,14 +1393,17 @@ hdrl_parallel_filter_image(const cpl_image * img,
         ky = cpl_mask_get_size_y(mask);
         kx = cpl_mask_get_size_x(mask);
     }
-    intptr_t hk = (ky / 2);
-    cpl_image * res = cpl_image_new(nx, ny, cpl_image_get_type(img));
-
     cpl_ensure(ky % 2 == 1, CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
     cpl_ensure(ny >= ky, CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
     cpl_ensure(nx >= kx, CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+
+    intptr_t hk = (ky / 2);
+    cpl_image * res = cpl_image_new(nx, ny, cpl_image_get_type(img));
+    /* make sure image has a bpm to avoid creation races later */
+    cpl_image_get_bpm(res);
+
     /* filter first half-kernel rows (needs full kernel image) */
-    {
+    if (hk) {
         const cpl_image * slice = image_const_row_view_create(img, 1, ky);
         cpl_image * slres = cpl_image_duplicate(slice);
         if (kernel) {
@@ -1278,13 +1419,13 @@ hdrl_parallel_filter_image(const cpl_image * img,
         image_const_row_view_delete(slice);
         cpl_image_delete(slres);
     }
-    intptr_t y;
+    intptr_t y = hk;
     const intptr_t s = 200;
     /* filter s sized row chunks needs one kernel overlap */
 HDRL_OMP(omp parallel for lastprivate(y) if (ny > s + ky))
     for (y = hk; y < ny - ky - (ny - ky) % s; y+=s) {
         intptr_t l = (y + 1) - hk;
-        intptr_t u = (y + 1 + s) + hk;
+        intptr_t u = (y + 1 + s) + hk - 1;
         const cpl_image * slice = image_const_row_view_create(img, l, u);
         cpl_image * slres = cpl_image_new(nx, u - l + 1,
                                           cpl_image_get_type(slice));
@@ -1295,14 +1436,14 @@ HDRL_OMP(omp parallel for lastprivate(y) if (ny > s + ky))
             cpl_image_filter_mask(slres, slice, mask, mode, border);
         }
         const cpl_image * slice2 =
-            image_const_row_view_create(slres, hk + 1, u - l - hk);
+            image_const_row_view_create(slres, hk + 1, hk + s);
         cpl_image_copy(res, slice2, 1, y + 1);
         image_const_row_view_delete(slice);
         image_const_row_view_delete(slice2);
         cpl_image_delete(slres);
     }
     /* filter remainder, needs half kernel overlap */
-    {
+    if (y + 1 - hk < ny) {
         const cpl_image * slice = image_const_row_view_create(img, y + 1 - hk, ny);
         cpl_image * slres = cpl_image_duplicate(slice);
         if (kernel) {
@@ -1322,6 +1463,145 @@ HDRL_OMP(omp parallel for lastprivate(y) if (ny > s + ky))
 }
 
 
+/**
+ * @brief
+ *    Convert between physical and world coordinates using multiple threads
+ *
+ * @param wcs       The input cpl_wcs structure
+ * @param from      The input coordinate matrix
+ * @param to        The output coordinate matrix
+ * @param status    The output status array
+ * @param transform The transformation mode
+ *
+ * @return  An appropriate error code
+ *
+ * @see cpl_wcs_convert()
+ *
+ * This function is a wrapper around cpl_wcs_convert using OpenMP for
+ * parallelization, see the cpl documentation for details on the functionality.
+ */
+    /* ---------------------------------------------------------------------------*/
+cpl_error_code
+hdrl_wcs_convert(const cpl_wcs *wcs, const cpl_matrix *from,
+                 cpl_matrix **to, cpl_array **status,
+                 cpl_wcs_trans_mode transform)
+{
+    size_t nr = cpl_matrix_get_nrow(from);
+    size_t nc = cpl_matrix_get_ncol(from);
+    const size_t s = 4000;
+    size_t i;
+    int * dstatus;
+    cpl_ensure_code(to, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(status, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(wcs, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(from, CPL_ERROR_NULL_INPUT);
+
+    *status = cpl_array_new(nr, CPL_TYPE_INT);
+    cpl_ensure_code(*status, CPL_ERROR_NULL_INPUT);
+    dstatus = cpl_array_get_data_int(*status);
+    *to = cpl_matrix_new(nr, nc);
+    cpl_error_code err = CPL_ERROR_NONE;
+
+HDRL_OMP(omp parallel for if (nr > s))
+    for (i = 0; i < nr; i+=s) {
+        cpl_matrix * lfrom = cpl_matrix_extract(from, i, 0, 1, 1, CPL_MIN(s, nr - i), nc);
+        cpl_matrix * lto = NULL;
+        cpl_array * lstatus = NULL;
+        cpl_error_code lerr = cpl_wcs_convert(wcs, lfrom, &lto, &lstatus, transform);
+        if (lto != NULL) {
+            cpl_matrix_copy(*to, lto, i, 0);
+        }
+        if (lstatus != NULL) {
+            memcpy(&dstatus[i], cpl_array_get_data_int(lstatus),
+                   cpl_array_get_size(lstatus) * sizeof(*dstatus));
+        }
+        cpl_array_delete(lstatus);
+        cpl_matrix_delete(lfrom);
+        cpl_matrix_delete(lto);
+        if (lerr != CPL_ERROR_NONE) {
+HDRL_OMP(omp critical(hdrlwcserror))
+            err = lerr;
+        }
+    }
+
+    if (err == CPL_ERROR_UNSUPPORTED_MODE) {
+        cpl_matrix_delete(*to);
+        *to = NULL;
+        cpl_array_delete(*status);
+        *status = NULL;
+    }
+
+    return cpl_error_set(cpl_func, err);
+}
+
+/* cpl_image_set_bpm from cpl 6.4 which we can't require yet for VLTSW 2014 */
+struct _cpl_image_ {
+    /* Size of the image in x and y */
+    cpl_size            nx, ny;
+    /* Type of the pixels used for the cpl_image */
+    cpl_type            type;
+    /* Pointer to pixel buffer as a 1d buffer */
+    void            *   pixels;
+    /* Bad Pixels mask */
+    cpl_mask        *   bpm;
+};
+
+cpl_mask * hcpl_image_set_bpm(cpl_image * self, cpl_mask * bpm) 
+{
+#if CPL_VERSION_CODE < CPL_VERSION(6, 4, 0)
+    cpl_mask * oldbpm;
+
+    cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, NULL);
+
+    oldbpm = self->bpm;
+    self->bpm = bpm; 
+ 
+    return oldbpm;
+#else
+    return cpl_image_set_bpm(self, bpm);
+#endif
+}
+
+double hcpl_gaussian_eval_2d(const cpl_array * self, double x, double y)
+{
+#if CPL_VERSION_CODE < CPL_VERSION(6, 4, 0)
+    cpl_errorstate prestate = cpl_errorstate_get();
+    const double B    = cpl_array_get_double(self, 0, NULL);
+    const double A    = cpl_array_get_double(self, 1, NULL);
+    const double R    = cpl_array_get_double(self, 2, NULL);
+    const double M_x  = cpl_array_get_double(self, 3, NULL);
+    const double M_y  = cpl_array_get_double(self, 4, NULL);
+    const double S_x  = cpl_array_get_double(self, 5, NULL);
+    const double S_y  = cpl_array_get_double(self, 6, NULL);
+
+    double value = 0.0; 
+
+    if (!cpl_errorstate_is_equal(prestate)) {
+        (void)cpl_error_set_where(cpl_func);
+    } else if (cpl_array_get_size(self) != 7) { 
+        (void)cpl_error_set(cpl_func, CPL_ERROR_ILLEGAL_INPUT);
+    } else if (fabs(R) < 1.0 && S_x != 0.0 && S_y != 0.0) {
+        const double x_n  = (x - M_x) / S_x; 
+        const double y_n  = (y - M_y) / S_y; 
+
+        value = B + A / (CPL_MATH_2PI * S_x * S_y * sqrt(1 - R * R)) *
+            exp(-0.5 / (1 - R * R) * ( x_n * x_n + y_n * y_n
+                                       - 2.0 * R * x_n * y_n));
+    } else if (fabs(R) > 1.0) {
+        (void)cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_OUTPUT,
+                                     "fabs(R=%g) > 1", R);
+    } else {
+        (void)cpl_error_set_message(cpl_func, CPL_ERROR_DIVISION_BY_ZERO,
+                                     "R=%g. Sigma=(%g, %g)", R, S_x, S_y);
+    }    
+
+    return value;
+#else
+    return cpl_gaussian_eval_2d(self, x, y);
+#endif
+}
+
+
 /** @endcond */
 
 /**@}*/
