@@ -188,6 +188,16 @@ void test_l2i_and_l2v(cpl_imagelist * data,
 
 void test_parameters(void)
 {
+    hdrl_parameter * hpar = hdrl_collapse_mean_parameter_create();
+    cpl_test(hdrl_collapse_parameter_is_mean(hpar));
+    hdrl_parameter_delete(hpar);
+    hpar = hdrl_collapse_median_parameter_create();
+    cpl_test(hdrl_collapse_parameter_is_median(hpar));
+    hdrl_parameter_delete(hpar);
+    hpar = hdrl_collapse_weighted_mean_parameter_create();
+    cpl_test(hdrl_collapse_parameter_is_weighted_mean(hpar));
+    hdrl_parameter_delete(hpar);
+
     cpl_test(hdrl_collapse_parameter_is_mean(HDRL_COLLAPSE_MEAN));
     cpl_test(!hdrl_collapse_parameter_is_mean(HDRL_COLLAPSE_MEDIAN));
     hdrl_parameter_delete((hdrl_parameter*)HDRL_COLLAPSE_MEAN);
@@ -493,7 +503,7 @@ int main(void)
                          (pair){0.0099473975744101273, HDRL_EPS_ERROR},
                          expect_contrib);
     }
-    /* test all values rejected list to image */
+    /* test all values rejected in line, list to image */
     {
         double v[] = {1, 2, 1, 3, 2};
         double e[] = {0.5, 0.7, 0.1, 1.0, 0.01};
@@ -541,6 +551,52 @@ int main(void)
                          expect_contrib_minmax);
         cpl_image_delete(expect_contrib_minmax);
     }
+    /* test all values rejected, list to image */
+    {
+        for (cpl_size i = 0; i < nz; i++) {
+            cpl_image * tmp = cpl_imagelist_get(data, i);
+            cpl_image_accept_all(tmp);
+            cpl_mask_not(cpl_image_get_bpm(tmp));
+        }
+        cpl_image_delete(expect_contrib);
+        expect_contrib = cpl_image_new_from_accepted(data);
+
+        hdrl_collapse_imagelist_to_image_t * img_meth[] = {
+            hdrl_collapse_imagelist_to_image_mean(),
+            hdrl_collapse_imagelist_to_image_sigclip(3., 3., 3),
+            hdrl_collapse_imagelist_to_image_weighted_mean(),
+            hdrl_collapse_imagelist_to_image_median(),
+            hdrl_collapse_imagelist_to_image_minmax(1., 1.),
+        };
+        hdrl_collapse_imagelist_to_vector_t * vec_meth[] = {
+            hdrl_collapse_imagelist_to_vector_mean(),
+            hdrl_collapse_imagelist_to_vector_sigclip(3., 3., 3),
+            hdrl_collapse_imagelist_to_vector_weighted_mean(),
+            hdrl_collapse_imagelist_to_vector_median(),
+            hdrl_collapse_imagelist_to_vector_minmax(1., 1.),
+        };
+
+        for (size_t i = 0; i < ARRAY_LEN(img_meth); i++) {
+            cpl_image * outimg_loc, *outerr_loc, *contrib_loc;
+            hdrl_imagelist_combine(data, errs, img_meth[i], &outimg_loc, &outerr_loc, &contrib_loc);
+            /* unlike cpl, don't emit error */
+            cpl_test_error(CPL_ERROR_NONE);
+
+            cpl_test_image_abs(contrib_loc, expect_contrib, 0);
+            cpl_test_eq(cpl_image_count_rejected(outimg_loc), nx *ny);
+            cpl_test_eq(cpl_image_count_rejected(outerr_loc), nx *ny);
+            cpl_image_delete(outimg_loc);
+            cpl_image_delete(outerr_loc);
+            cpl_image_delete(contrib_loc);
+
+            /* also check vector variant */
+            test_l2i_and_l2v(data, errs, img_meth[i], vec_meth[i], 1, 1,
+                             (pair){NAN, HDRL_EPS_DATA},
+                             (pair){NAN, HDRL_EPS_ERROR},
+                             expect_contrib);
+            cpl_test_error(CPL_ERROR_NONE);
+        }
+    }
     /* test median error propagation with rejects, only makes sense on uniform
      * errors as the scaling relies on gaussian errors */
     {
