@@ -105,24 +105,24 @@ static cpl_error_code hdrl_clip_kappa_sigma_test(void)
     }
 
     {
-        /* iqr sigma ~1.5 median 6, check that 2 and 10 which are closer to
-         * 6 -+ 3 than 4 and 6 but beyond the k*sig limit are not included */
-        double values[] = {2.5, 6., 4., 6., 4., 6., 4., 6., 10.};
+        /* MAD sigma ~3 median 6, check that 1.5 and 10.5 which are closer to
+         * 6 -+ 3 * 1.5 than 4 and 6 but beyond the k*sig limit are not included */
+        double values[] = {1.5, 6., 4., 6., 4., 6., 4., 6., 10.5};
         const int n = ARRAY_LEN(values);
 
         cpl_vector * data = cpl_vector_wrap(n, values);
         cpl_vector * errors = cpl_vector_new(n);
         cpl_vector_fill(errors, 1);
 
-        hdrl_kappa_sigma_clip(data, errors, 2., 2., 1, CPL_FALSE,
+        hdrl_kappa_sigma_clip(data, errors, 1.5, 1.5, 1, CPL_FALSE,
                               &omean, &omean_err, &naccepted,
                               &rej_low, &rej_high);
         cpl_test_error(CPL_ERROR_NONE);
 
         cpl_test_rel(omean, (4. * 3 + 6 * 4.) / 7., 0.001);
         cpl_test_rel(omean_err, 1 / sqrt(n - 2), 0.001);
-        cpl_test_rel(rej_low, 3, 0.02);
-        cpl_test_rel(rej_high, 9, 0.02);
+        cpl_test_rel(rej_low, 1.55, 0.02);
+        cpl_test_rel(rej_high, 10.44, 0.02);
         cpl_test_eq(naccepted, n - 2);
 
         cpl_vector_unwrap(data);
