@@ -31,7 +31,7 @@
 
 mosca::vector_cubicspline::vector_cubicspline() :
 m_bspline_workspace(NULL), m_covar(NULL), m_spline_coeffs(NULL), m_basis(NULL),
-m_x_max_knot(0.), m_x_min_knot(0.)
+m_x_min_knot(0.), m_x_max_knot(0.)
 {
 }
 
