@@ -416,13 +416,13 @@ static int vimos_calib_create(cpl_plugin *plugin)
 
     /*
      * Smooth box radius for flat field along spatial direction
-     * (if s_knots < 0)
+     * (if s_degree < 0)
      */
 
     p = cpl_parameter_new_value("vimos.vmmoscalib.sradius",
                                 CPL_TYPE_INT,
                                 "Smooth box radius for flat field along "
-                                "spatial direction (used if s_knots < 0)",
+                                "spatial direction (used if s_degree < 0)",
                                 "vimos.vmmoscalib",
                                 -1);
     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "sradius");
