@@ -64,6 +64,7 @@
 #include "vmextincttable.h"
 #include "vmspecphottable.h"
 #include "vmcpl.h"
+#include "vimos_dfs.h"
 
 
 static cxint vmifuscience(PilSetOfFrames *);
@@ -762,6 +763,13 @@ vmifuscience(PilSetOfFrames *sof)
 
   /*
    * Now go for the bias subtraction
+   * 
+   * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+   * which is the default. Since the overscan region in the master bias has 
+   * been created with growOverscans, there is some extra noise added here,
+   * but this can probably be ignored. A better approach is to use the method
+   * used in the MOS pipeline: subtract overscan for each individual frame,
+   * then subtract bias.
    */
 
   cpl_msg_info(task, "Bias removal...");
@@ -1020,7 +1028,7 @@ vmifuscience(PilSetOfFrames *sof)
     }
 
     if (extension == 1) {
-        strlower(strcpy(outTraceName, outTraceTag));
+        vmstrlower(strcpy(outTraceName, outTraceTag));
         strcat(outTraceName, ".fits");
         error = cpl_table_save(fcoeff, NULL, NULL, outTraceName, CPL_IO_DEFAULT);
         if (error)
@@ -1105,7 +1113,7 @@ vmifuscience(PilSetOfFrames *sof)
 #endif
 
       if (extension == 1) {
-          strlower(strcpy(outIdsName, outIdsTag));
+          vmstrlower(strcpy(outIdsName, outIdsTag));
           strcat(outIdsName, ".fits");
           error = cpl_table_save(ids, NULL, NULL, outIdsName, CPL_IO_DEFAULT);
           if (error)
@@ -1119,7 +1127,14 @@ vmifuscience(PilSetOfFrames *sof)
             cpl_msg_error(task, "Cannot extend local product file %s!",
                           outIdsName);
       }
+      outputFrame = newPilFrame(outIdsName, outIdsTag);
 
+      pilFrmSetType(outputFrame, PIL_FRAME_TYPE_PRODUCT);
+      pilFrmSetFormat(outputFrame, PIL_FRAME_FORMAT_IMAGE);
+      pilFrmSetProductLevel(outputFrame, PIL_PRODUCT_LEVEL_SECONDARY);
+      pilFrmSetProductType(outputFrame, PIL_PRODUCT_TYPE_REDUCED);
+
+      pilSofInsert(sof, outputFrame);
     }
 
     cpl_msg_info(task, "Resample scientific spectra at constant wavelength "
@@ -1142,15 +1157,6 @@ vmifuscience(PilSetOfFrames *sof)
   cpl_image_unwrap(science);
 
 
-  outputFrame = newPilFrame(outIdsName, outIdsTag);
-
-  pilFrmSetType(outputFrame, PIL_FRAME_TYPE_PRODUCT);
-  pilFrmSetFormat(outputFrame, PIL_FRAME_FORMAT_IMAGE);
-  pilFrmSetProductLevel(outputFrame, PIL_PRODUCT_LEVEL_SECONDARY);
-  pilFrmSetProductType(outputFrame, PIL_PRODUCT_TYPE_REDUCED);
-
-  pilSofInsert(sof, outputFrame);
-
   outputFrame = newPilFrame(outTraceName, outTraceTag);
   
   pilFrmSetType(outputFrame, PIL_FRAME_TYPE_PRODUCT);
@@ -1298,6 +1304,10 @@ vmifuscience(PilSetOfFrames *sof)
   deleteSetOfDescriptors(&(sciFov->descs), "ESO PRO ZERO*");
   deleteSetOfDescriptors(&(sciFov->descs), "CD1_*");  
   deleteSetOfDescriptors(&(sciFov->descs), "CD2_*");  
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(sciFov->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(sciFov->descs), "GCOUNT");  
 
   data = (float *)cpl_image_get_data(extracted);
   sciExtracted = newImage(xlen, ylen, data);
@@ -1315,6 +1325,10 @@ vmifuscience(PilSetOfFrames *sof)
   deleteSetOfDescriptors(&(sciExtracted->descs), "ESO PRO ZERO*");
   deleteSetOfDescriptors(&(sciExtracted->descs), "CD1_*");  
   deleteSetOfDescriptors(&(sciExtracted->descs), "CD2_*");  
+  //These keywords shouldn't be present in the input raw, but we nevertheless
+  //remove them in case they are present (See PIPE-5422)
+  deleteSetOfDescriptors(&(sciExtracted->descs), "PCOUNT");  
+  deleteSetOfDescriptors(&(sciExtracted->descs), "GCOUNT");  
 
   deleteImage(scienceImage);
 
@@ -1653,7 +1667,7 @@ vmifuscience(PilSetOfFrames *sof)
    * update the set of frames.
    */
 
-  strlower(strcpy(reducedName, reducedTag));
+  vmstrlower(strcpy(reducedName, reducedTag));
   strcat(reducedName, ".fits");
 
   if (createFitsImage(reducedName, sciExtracted, reducedTag)) {
@@ -1682,7 +1696,7 @@ vmifuscience(PilSetOfFrames *sof)
   deleteImage(sciExtracted);
   cpl_image_delete(extracted);
 
-  strlower(strcpy(fovName, fovTag));
+  vmstrlower(strcpy(fovName, fovTag));
   strcat(fovName, ".fits");
 
   if (createFitsImage(fovName, sciFov, fovTag)) {
@@ -1711,7 +1725,7 @@ vmifuscience(PilSetOfFrames *sof)
 
   if (calibrateFlux) {
 
-    strlower(strcpy(reducedName, fluxReducedTag));
+    vmstrlower(strcpy(reducedName, fluxReducedTag));
     strcat(reducedName, ".fits");
 
     if (createFitsImage(reducedName, sciFluxExtracted, fluxReducedTag)) {
