@@ -63,6 +63,7 @@
 #include "vmimgutils.h"
 #include "vmqcutils.h"
 #include "vmcpl.h"
+#include "vimos_dfs.h"
 
 static int vmimobsjitter_create(cpl_plugin *);
 static int vmimobsjitter_exec(cpl_plugin *);
@@ -1493,6 +1494,17 @@ vmimobsjitter(PilSetOfFrames *sof)
             biasImage = resizedBias;
         }
 
+        /*
+         * Now go for the bias subtraction
+         * 
+         * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+         * which is the default. Since the overscan region in the master bias has 
+         * been created with growOverscans, there is some extra noise added here,
+         * but this can probably be ignored. A better approach is to use the method
+         * used in the MOS pipeline: subtract overscan for each individual frame,
+         * then subtract bias.
+         */
+
         if (VmSubBias(rawImage, biasImage, biasMethod) == EXIT_FAILURE) {
             cpl_msg_error(task, "Bias correction failed!");
 
@@ -1789,7 +1801,7 @@ vmimobsjitter(PilSetOfFrames *sof)
             double  average, newAverage, tolerance, delta;
             double  nSigma = 1;
             float   value, minvalue;
-            float  *buffer = pil_malloc(ngals * sizeof(float));
+            float  *buffer = cpl_malloc(ngals * sizeof(float));
             float  *profile = NULL;
             float  *triprofile = NULL;
             float   meanbin = 1.0; /* Mean no. of objects per theta bin */
@@ -1968,8 +1980,8 @@ vmimobsjitter(PilSetOfFrames *sof)
 
             nbin = 180 / binsize;
 
-            profile = (float *)pil_calloc(nbin, sizeof(float));
-            triprofile = (float *)pil_calloc(3 * nbin, sizeof(float));
+            profile = (float *)cpl_calloc(nbin, sizeof(float));
+            triprofile = (float *)cpl_calloc(3 * nbin, sizeof(float));
 
             for (i = 0; i < ngals; i++) {
                 if (stellarity[i] > starIndex && flags[i] == 0) {
@@ -2271,8 +2283,8 @@ vmimobsjitter(PilSetOfFrames *sof)
 
                     nbin = 180 / binsize;
 
-                    profile = (float *)pil_calloc(nbin, sizeof(float));
-                    triprofile = (float *)pil_calloc(3 * nbin, sizeof(float));
+                    profile = (float *)cpl_calloc(nbin, sizeof(float));
+                    triprofile = (float *)cpl_calloc(3 * nbin, sizeof(float));
 
                     for (i = 0; i < ngals; i++) {
                         if (xImage[i] >= xStart && xImage[i] < xEnd) {
@@ -2355,7 +2367,7 @@ vmimobsjitter(PilSetOfFrames *sof)
             }
 
 
-            pil_free(buffer);
+            cpl_free(buffer);
 
 
             /* QC.MAGLIM */
@@ -2371,7 +2383,7 @@ vmimobsjitter(PilSetOfFrames *sof)
             for (j = 1; j < 10; j++) {
                 mbinsz = j / 10.0;
                 nbin = (magend - magbeg) / mbinsz;
-                ibuffer = pil_calloc(nbin, sizeof(int));
+                ibuffer = cpl_calloc(nbin, sizeof(int));
                 for (i = 0; i < ngals; i++) {
                     if (stellarity[i] > starIndex && flags[i] == 0) {
                         pos = floor((magnitude[i] - magbeg) / mbinsz);
@@ -2389,7 +2401,7 @@ vmimobsjitter(PilSetOfFrames *sof)
                     }
                 }
 
-                pil_free(ibuffer);
+                cpl_free(ibuffer);
 
                 if (ivalue > minstar) {
                     maglim = magbeg + mbinsz * maxpos + mbinsz / 2.0;
@@ -2464,7 +2476,7 @@ vmimobsjitter(PilSetOfFrames *sof)
         nsec = 10;  /* Divide region sides by this number to define subregions */
 
         npix = (winSizeX / nsec) * (winSizeY / nsec);
-        buffer = pil_calloc(nsec * nsec, sizeof(float));
+        buffer = cpl_calloc(nsec * nsec, sizeof(float));
         j = 0;
 
         for (sx = 0; sx < winSizeX; sx += winSizeX / nsec) {
@@ -2473,18 +2485,18 @@ vmimobsjitter(PilSetOfFrames *sof)
                         winSizeX / nsec, winSizeY / nsec);
                 buffer[j] = medianPixelvalue(subregion, npix);
                 j++;
-                pil_free(subregion);
+                cpl_free(subregion);
             }
         }
-        pil_free(region);
+        cpl_free(region);
 
         nvalues = 10;  /* Number of lowest values to consider */
-        lvalues = pil_calloc(nvalues, sizeof(float));
+        lvalues = cpl_calloc(nvalues, sizeof(float));
 
         for (i = 0; i < nvalues; i++)
             lvalues[i] = kthSmallest(buffer, j, i);
 
-        pil_free(buffer);
+        cpl_free(buffer);
 
         meanValue = computeAverageFloat(lvalues, nvalues);
 
@@ -2558,7 +2570,7 @@ vmimobsjitter(PilSetOfFrames *sof)
 
     productTag = (char *)pilTrnGetCategory("redImgScience");
 
-    strlower(strcpy(productName, productTag));
+    vmstrlower(strcpy(productName, productTag));
     strcat(productName, ".fits");
 
 
@@ -2644,7 +2656,7 @@ vmimobsjitter(PilSetOfFrames *sof)
 
     productTag = (char *)pilTrnGetCategory("GalaxyTable");
 
-    strlower(strcpy(productName, productTag));
+    vmstrlower(strcpy(productName, productTag));
     /* strcat(productName, ".TFITS"); */
     strcat(productName, ".fits");
 
@@ -2683,7 +2695,7 @@ vmimobsjitter(PilSetOfFrames *sof)
 
         productTag = (char *)pilTrnGetCategory("ImgFringes");
 
-        strlower(strcpy(productName, productTag));
+        vmstrlower(strcpy(productName, productTag));
         strcat(productName, ".fits");
 
 
