@@ -67,6 +67,7 @@
 #include "vmifuutils.h"
 #include "vmqcutils.h"
 #include "vmcpl.h"
+#include "vimos_dfs.h"
 
 
 static cxint vmspflat(PilSetOfFrames *);
@@ -750,7 +751,7 @@ vmspflat(PilSetOfFrames *sof)
   int         goodFrames;
 
   int         i, j, k;
-  int         cleanBadPixel, cleanCosmic, validateFrames, applyQc;
+  int         cleanBadPixel, cleanCosmic, validateFrames;
   int         singleFrames, computeQC;
   int         error;
 
@@ -1013,7 +1014,7 @@ vmspflat(PilSetOfFrames *sof)
    * final product (TO BE IMPLEMENTED).
    */
 
-  applyQc = pilDfsDbGetBool(parameter, "ApplyQC", 0);
+  //applyQc = pilDfsDbGetBool(parameter, "ApplyQC", 0);
 
 
   /*
@@ -1214,7 +1215,7 @@ vmspflat(PilSetOfFrames *sof)
 
   error = 1;
 
-  if ((flatList = (VimosImage **)pil_calloc(flatCount, sizeof(VimosImage*)))) {
+  if ((flatList = (VimosImage **)cpl_calloc(flatCount, sizeof(VimosImage*)))) {
 
     error = 0;
     currFrame = pilSofLookupNext(sof, flatCategory);
@@ -1232,7 +1233,7 @@ vmspflat(PilSetOfFrames *sof)
         }
         if (flatList[i])
           closeFitsImage(flatList[i], 0);
-        pil_free(flatList);
+        cpl_free(flatList);
         break;
       }
       currFrame = pilSofLookupNext(sof, NULL);
@@ -1264,7 +1265,7 @@ vmspflat(PilSetOfFrames *sof)
       closeFitsImage(flatList[i], 0);
       deleteImage(flatList[i]);
     }
-    pil_free(flatList);
+    cpl_free(flatList);
     deleteImage(biasImage);
     deleteImage(darkImage);
     deleteTable(ccdTable);
@@ -1304,7 +1305,7 @@ vmspflat(PilSetOfFrames *sof)
       closeFitsImage(flatList[i], 0);
       deleteImage(flatList[i]);
     }
-    pil_free(flatList);
+    cpl_free(flatList);
     deleteImage(biasImage);
     deleteImage(darkImage);
     deleteTable(ccdTable);
@@ -1330,7 +1331,7 @@ vmspflat(PilSetOfFrames *sof)
       closeFitsImage(flatList[i], 0);
       deleteImage(flatList[i]);
     }
-    pil_free(flatList);
+    cpl_free(flatList);
     deleteExtractionTable(extractionTable);
     deleteImage(biasImage);
     deleteImage(darkImage);
@@ -1370,8 +1371,8 @@ vmspflat(PilSetOfFrames *sof)
           closeFitsImage(flatList[j], 0);
           deleteImage(flatList[j]);
         }
-        pil_free(flatList);
-        pil_free(flatCountShutPos);
+        cpl_free(flatList);
+        cpl_free(flatCountShutPos);
         deleteExtractionTable(extractionTable);
         deleteImage(biasImage);
         deleteImage(darkImage);
@@ -1395,7 +1396,7 @@ vmspflat(PilSetOfFrames *sof)
   * Prepare storage for stacked frames
   */
 
-  combList = (VimosImage **)pil_calloc(shutPosCount, sizeof(VimosImage *));
+  combList = (VimosImage **)cpl_calloc(shutPosCount, sizeof(VimosImage *));
 
   if (!combList) {
     cpl_msg_error(task, "Not enough memory");
@@ -1403,8 +1404,8 @@ vmspflat(PilSetOfFrames *sof)
       closeFitsImage(flatList[i], 0);
       deleteImage(flatList[i]);
     }
-    pil_free(flatList);
-    pil_free(flatCountShutPos);
+    cpl_free(flatList);
+    cpl_free(flatCountShutPos);
     deleteExtractionTable(extractionTable);
     deleteImage(biasImage);
     deleteImage(darkImage);
@@ -1441,12 +1442,12 @@ vmspflat(PilSetOfFrames *sof)
           closeFitsImage(flatList[k], 0);
           deleteImage(flatList[k]);
         }
-        pil_free(flatList);
+        cpl_free(flatList);
 
         for (i = 0; i < shutPosCount; i++)
           deleteImage(combList[i]);
-        pil_free(combList);
-        pil_free(flatCountShutPos);
+        cpl_free(combList);
+        cpl_free(flatCountShutPos);
         deleteExtractionTable(extractionTable);
         deleteImage(biasImage);
         deleteImage(darkImage);
@@ -1465,7 +1466,7 @@ vmspflat(PilSetOfFrames *sof)
     error = 1;
 
     if ((flatGain = 
-        (float *)pil_calloc(flatCountShutPos[shutter], sizeof(float)))) {
+        (float *)cpl_calloc(flatCountShutPos[shutter], sizeof(float)))) {
       error = 0;
       for (i = 0; i < flatCountShutPos[shutter]; i++) {
         gain = getMeanGainFactor(flatSubList[i]);
@@ -1477,7 +1478,7 @@ vmspflat(PilSetOfFrames *sof)
           cpl_msg_error(task, "Wrong or no gain factor for flat field %d/%d "
                       "of shutter position %d/%d", i + 1, 
                       flatCountShutPos[shutter], shutter + 1, shutPosCount);
-          pil_free(flatGain);
+          cpl_free(flatGain);
           break;
         }
       }
@@ -1492,11 +1493,11 @@ vmspflat(PilSetOfFrames *sof)
         closeFitsImage(flatList[k], 0);
         deleteImage(flatList[k]);
       }
-      pil_free(flatList);
+      cpl_free(flatList);
       for (i = 0; i < shutPosCount; i++)
         deleteImage(combList[i]);
-      pil_free(combList);
-      pil_free(flatCountShutPos);
+      cpl_free(combList);
+      cpl_free(flatCountShutPos);
       deleteExtractionTable(extractionTable);
       deleteImage(biasImage);
       deleteImage(darkImage);
@@ -1514,7 +1515,7 @@ vmspflat(PilSetOfFrames *sof)
     */
 
     if ((flatRon = 
-        (float *)pil_calloc(flatCountShutPos[shutter], sizeof(float)))) {
+        (float *)cpl_calloc(flatCountShutPos[shutter], sizeof(float)))) {
       for (i = 0; i < flatCountShutPos[shutter]; i++) {
         flatRon[i] = computeAverageRon(flatSubList[i]);
       }
@@ -1528,12 +1529,12 @@ vmspflat(PilSetOfFrames *sof)
         closeFitsImage(flatList[k], 0);
         deleteImage(flatList[k]);
       }
-      pil_free(flatList);
+      cpl_free(flatList);
       for (i = 0; i < shutPosCount; i++)
         deleteImage(combList[i]);
-      pil_free(combList);
-      pil_free(flatCountShutPos);
-      pil_free(flatGain);
+      cpl_free(combList);
+      cpl_free(flatCountShutPos);
+      cpl_free(flatGain);
       deleteExtractionTable(extractionTable);
       deleteImage(biasImage);
       deleteImage(darkImage);
@@ -1544,9 +1545,16 @@ vmspflat(PilSetOfFrames *sof)
     }
 
 
-   /*
-    * Now go for the bias subtraction
-    */
+    /*
+     * Now go for the bias subtraction
+     * 
+     * NOTE: This includes the overscan subtraction if biasMethod = BIAS_ZMASTER,
+     * which is the default. Since the overscan region in the master bias has 
+     * been created with growOverscans, there is some extra noise added here,
+     * but this can probably be ignored. A better approach is to use the method
+     * used in the MOS pipeline: subtract overscan for each individual frame,
+     * then subtract bias.
+     */
 
     cpl_msg_info(task, "Bias removal for shutter position %d/%d frames...",
                shutter + 1, shutPosCount);
@@ -1562,13 +1570,13 @@ vmspflat(PilSetOfFrames *sof)
           closeFitsImage(flatList[k], 0);
           deleteImage(flatList[k]);
         }
-        pil_free(flatList);
+        cpl_free(flatList);
         for (i = 0; i < shutPosCount; i++)
           deleteImage(combList[i]);
-        pil_free(combList);
-        pil_free(flatCountShutPos);
-        pil_free(flatGain);
-        pil_free(flatRon);
+        cpl_free(combList);
+        cpl_free(flatCountShutPos);
+        cpl_free(flatGain);
+        cpl_free(flatRon);
         deleteExtractionTable(extractionTable);
         deleteImage(biasImage);
         deleteImage(darkImage);
@@ -1601,13 +1609,13 @@ vmspflat(PilSetOfFrames *sof)
             closeFitsImage(flatList[k], 0);
             deleteImage(flatList[k]);
           }
-          pil_free(flatList);
+          cpl_free(flatList);
           for (i = 0; i < shutPosCount; i++)
             deleteImage(combList[i]);
-          pil_free(combList);
-          pil_free(flatCountShutPos);
-          pil_free(flatGain);
-          pil_free(flatRon);
+          cpl_free(combList);
+          cpl_free(flatCountShutPos);
+          cpl_free(flatGain);
+          cpl_free(flatRon);
           deleteExtractionTable(extractionTable);
           deleteImage(darkImage);
           deleteTable(ccdTable);
@@ -1626,7 +1634,7 @@ vmspflat(PilSetOfFrames *sof)
     */
 
     if ((flatLevel = 
-        (float *)pil_calloc(flatCountShutPos[shutter], sizeof(float)))) {
+        (float *)cpl_calloc(flatCountShutPos[shutter], sizeof(float)))) {
 
      /* FIXME:
       * Here the "flag" is used to switch between determining the
@@ -1715,13 +1723,13 @@ vmspflat(PilSetOfFrames *sof)
               closeFitsImage(flatList[k], 0);
               deleteImage(flatList[k]);
             }
-            pil_free(flatList);
+            cpl_free(flatList);
             for (i = 0; i < shutPosCount; i++)
               deleteImage(combList[i]);
-            pil_free(combList);
-            pil_free(flatCountShutPos);
-            pil_free(flatGain);
-            pil_free(flatRon);
+            cpl_free(combList);
+            cpl_free(flatCountShutPos);
+            cpl_free(flatGain);
+            cpl_free(flatRon);
             deleteExtractionTable(extractionTable);
             deleteImage(darkImage);
             deleteTable(ccdTable);
@@ -1733,7 +1741,7 @@ vmspflat(PilSetOfFrames *sof)
           flatLevel[i] = (float)computeAverageFloat(extrData, 
                                                     windowDimen * windowDimen);
 
-          pil_free(extrData);
+          cpl_free(extrData);
 
           cpl_msg_info(task, "Level of flat field %d/%d of shutter position "
                      "%d/%d is %10.4f ADU", i + 1, flatCountShutPos[shutter], 
@@ -1763,13 +1771,13 @@ vmspflat(PilSetOfFrames *sof)
         closeFitsImage(flatList[k], 0);
         deleteImage(flatList[k]);
       }
-      pil_free(flatList);
+      cpl_free(flatList);
       for (i = 0; i < shutPosCount; i++)
         deleteImage(combList[i]);
-      pil_free(combList);
-      pil_free(flatCountShutPos);
-      pil_free(flatGain);
-      pil_free(flatRon);
+      cpl_free(combList);
+      cpl_free(flatCountShutPos);
+      cpl_free(flatGain);
+      cpl_free(flatRon);
       deleteExtractionTable(extractionTable);
       deleteTable(ccdTable);
       deleteTable(grismTable);
@@ -1800,14 +1808,14 @@ vmspflat(PilSetOfFrames *sof)
             closeFitsImage(flatList[k], 0);
             deleteImage(flatList[k]);
           }
-          pil_free(flatList);
+          cpl_free(flatList);
           for (i = 0; i < shutPosCount; i++)
             deleteImage(combList[i]);
-          pil_free(combList);
-          pil_free(flatCountShutPos);
-          pil_free(flatGain);
-          pil_free(flatRon);
-          pil_free(flatLevel);
+          cpl_free(combList);
+          cpl_free(flatCountShutPos);
+          cpl_free(flatGain);
+          cpl_free(flatRon);
+          cpl_free(flatLevel);
           deleteExtractionTable(extractionTable);
           deleteTable(ccdTable);
           deleteTable(grismTable);
@@ -1820,8 +1828,8 @@ vmspflat(PilSetOfFrames *sof)
       }
     }
 
-    pil_free(flatRon);
-    pil_free(flatGain);
+    cpl_free(flatRon);
+    cpl_free(flatGain);
 
 
     if (validateFrames) {
@@ -1857,7 +1865,7 @@ vmspflat(PilSetOfFrames *sof)
 
 /*
         for (i = goodFrames; i < flatCountShutPos[shutter]; i++) {
-          pil_free(flatSubList[i]->data);
+          cpl_free(flatSubList[i]->data);
           flatSubList[i]->data = NULL;
         }
 */
@@ -1870,7 +1878,7 @@ vmspflat(PilSetOfFrames *sof)
       goodFrames = flatCountShutPos[shutter];
 
 
-    pil_free(flatLevel);
+    cpl_free(flatLevel);
 
    /*
     * Stack the surviving flat field frames if requested and if possible,
@@ -1896,11 +1904,11 @@ vmspflat(PilSetOfFrames *sof)
         closeFitsImage(flatList[k], 0);
         deleteImage(flatList[k]);
       }
-      pil_free(flatList);
+      cpl_free(flatList);
       for (i = 0; i < shutPosCount; i++)
         deleteImage(combList[i]);
-      pil_free(combList);
-      pil_free(flatCountShutPos);
+      cpl_free(combList);
+      cpl_free(flatCountShutPos);
       deleteExtractionTable(extractionTable);
       deleteTable(ccdTable);
       deleteTable(grismTable);
@@ -1917,11 +1925,11 @@ vmspflat(PilSetOfFrames *sof)
         closeFitsImage(flatList[k], 0);
         deleteImage(flatList[k]);
       }
-      pil_free(combList);
+      cpl_free(combList);
       for (i = 0; i < flatCount; i++)
         deleteImage(flatList[i]);
-      pil_free(flatList);
-      pil_free(flatCountShutPos);
+      cpl_free(flatList);
+      cpl_free(flatCountShutPos);
       deleteExtractionTable(extractionTable);
       deleteTable(ccdTable);
       deleteTable(grismTable);
@@ -1976,15 +1984,15 @@ vmspflat(PilSetOfFrames *sof)
       if (error) {
         for (i = 0; i < shutPosCount; i++)
           deleteImage(combList[i]);
-        pil_free(combList);
+        cpl_free(combList);
         for (j = 0, k = 0; k < shutter; k++)
           j += flatCountShutPos[k];
         for (k = j; k < flatCount; k++) {
           closeFitsImage(flatList[k], 0);
           deleteImage(flatList[k]);
         }
-        pil_free(flatList);
-        pil_free(flatCountShutPos);
+        cpl_free(flatList);
+        cpl_free(flatCountShutPos);
         deleteExtractionTable(extractionTable);
         deleteTable(ccdTable);
         deleteTable(grismTable);
@@ -2013,15 +2021,15 @@ vmspflat(PilSetOfFrames *sof)
 
   deleteImage(biasImage);
   deleteImage(darkImage);
-  pil_free(flatList);
-  pil_free(flatCountShutPos);
+  cpl_free(flatList);
+  cpl_free(flatCountShutPos);
 
   if (!(stackImages = VmSpStackFF(combList,
                                   shutPosCount, extractionTable, fuzz))) {
     cpl_msg_error(task, "Failure stacking flat fields");
     for (i = 0; i < shutPosCount; i++)
       deleteImage(combList[i]);
-    pil_free(combList);
+    cpl_free(combList);
     deleteExtractionTable(extractionTable);
     deleteTable(ccdTable);
     deleteTable(grismTable);
@@ -2031,7 +2039,7 @@ vmspflat(PilSetOfFrames *sof)
 
   for (i = 0; i < shutPosCount; i++)
     deleteImage(combList[i]);
-  pil_free(combList);
+  cpl_free(combList);
 
 
  /*
@@ -2047,7 +2055,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteIfuTable(ifuTable);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2078,7 +2086,7 @@ vmspflat(PilSetOfFrames *sof)
     if (error) {
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       deleteExtractionTable(extractionTable);
       deleteTable(ccdTable);
       deleteTable(grismTable);
@@ -2099,7 +2107,7 @@ vmspflat(PilSetOfFrames *sof)
                   "polynomials on stacked flat field");
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       deleteExtractionTable(extractionTable);
       deleteTable(grismTable);
       deleteIfuTable(ifuTable);
@@ -2194,7 +2202,7 @@ vmspflat(PilSetOfFrames *sof)
                   "polynomials on stacked flat field");
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       deleteExtractionTable(extractionTable);
       deleteTable(grismTable);
       deleteIfuTable(ifuTable);
@@ -2207,7 +2215,7 @@ vmspflat(PilSetOfFrames *sof)
                   "for stacked flat field");
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       deleteExtractionTable(extractionTable);
       deleteTable(grismTable);
       deleteIfuTable(ifuTable);
@@ -2219,7 +2227,7 @@ vmspflat(PilSetOfFrames *sof)
                   "for stacked flat field");
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       deleteExtractionTable(extractionTable);
       deleteTable(grismTable);
       deleteIfuTable(ifuTable);
@@ -2250,7 +2258,7 @@ vmspflat(PilSetOfFrames *sof)
       cpl_msg_error(task, "Failure upgrading the extraction table");
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       deleteTable(grismTable);
       deleteIfuTable(ifuTable);
       return EXIT_FAILURE;
@@ -2268,7 +2276,7 @@ vmspflat(PilSetOfFrames *sof)
         cpl_msg_error(task, "Failure upgrading the extraction table");
         deleteImage(stackImages[0]);
         deleteImage(stackImages[1]);
-        pil_free(stackImages);
+        cpl_free(stackImages);
         deleteTable(grismTable);
         deleteIfuTable(ifuTable);
         return EXIT_FAILURE;
@@ -2299,7 +2307,7 @@ vmspflat(PilSetOfFrames *sof)
       deleteExtractionTable(extractionTable);
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       return EXIT_FAILURE;
     }
   }
@@ -2390,7 +2398,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteImage(masterFlat);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2411,7 +2419,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteImage(masterFlat);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2425,7 +2433,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteImage(masterFlat);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2444,7 +2452,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteImage(masterFlat);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2525,7 +2533,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteImage(masterFlat);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2588,7 +2596,7 @@ vmspflat(PilSetOfFrames *sof)
       deleteImage(stackImages[0]);
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       return EXIT_FAILURE;
     }
 
@@ -2652,7 +2660,7 @@ vmspflat(PilSetOfFrames *sof)
       deleteImage(masterFlat);
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       return EXIT_FAILURE;
     }
 
@@ -2673,7 +2681,7 @@ vmspflat(PilSetOfFrames *sof)
   * Output the primary product: the spectral master flat field.
   */
 
-  strlower(strcpy(masterFlatName, masterCategory));
+  vmstrlower(strcpy(masterFlatName, masterCategory));
   strcat(masterFlatName, ".fits");
 
   if (createFitsImage(masterFlatName, masterFlat, masterCategory)) {
@@ -2691,7 +2699,7 @@ vmspflat(PilSetOfFrames *sof)
     deleteImage(masterFlat);
     deleteImage(stackImages[0]);
     deleteImage(stackImages[1]);
-    pil_free(stackImages);
+    cpl_free(stackImages);
     return EXIT_FAILURE;
   }
 
@@ -2703,7 +2711,7 @@ vmspflat(PilSetOfFrames *sof)
     * This is new...
     */
 
-    strlower(strcpy(masterFlatName, mosCombCategory));
+    vmstrlower(strcpy(masterFlatName, mosCombCategory));
     strcat(masterFlatName, ".fits");
 
     if (createFitsImage(masterFlatName, stackImages[0], mosCombCategory)) {
@@ -2722,7 +2730,7 @@ vmspflat(PilSetOfFrames *sof)
       deleteImage(masterFlat);
       deleteImage(stackImages[0]);
       deleteImage(stackImages[1]);
-      pil_free(stackImages);
+      cpl_free(stackImages);
       return EXIT_FAILURE;
     }
   }
@@ -2734,7 +2742,7 @@ vmspflat(PilSetOfFrames *sof)
     * This is the zero order image
     */
 
-    strlower(strcpy(zeroOrderName, zeroOrderCategory));
+    vmstrlower(strcpy(zeroOrderName, zeroOrderCategory));
     strcat(zeroOrderName, ".fits");
 
     if (createFitsImage(zeroOrderName, stackImages[1], zeroOrderCategory)) {
@@ -2753,7 +2761,7 @@ vmspflat(PilSetOfFrames *sof)
 
   deleteImage(stackImages[0]);
   deleteImage(stackImages[1]);
-  pil_free(stackImages);
+  cpl_free(stackImages);
 
   if ((pafFileName = createSpectralDistModelsPAF(masterFlat->descs, namePAF))
                                                          != NULL) {
@@ -2765,7 +2773,7 @@ vmspflat(PilSetOfFrames *sof)
     pilFrmSetProductType(outputFrame, PIL_PRODUCT_TYPE_REDUCED);
 
     pilSofInsert(sof, outputFrame);
-    pil_free(pafFileName);
+    cpl_free(pafFileName);
   }
   else
     cpl_msg_error(task, "Cannot create local product file %s!", namePAF);
