@@ -38,7 +38,7 @@
 
 #include <cpl.h>
 #include <fors_paf.h>
-#include <fors_dfs.h>
+#include <vimos_dfs.h>
 #include <fors_qc.h>
 
 #define DICT_LINE_LENGTH    (80)
