@@ -2031,7 +2031,7 @@ int vimos_calib_impl(cpl_frameset *frameset, cpl_parameterlist *parlist)
     for(size_t ised = 0 ; ised < sed_norm.size();ised++)
     {
         std::ostringstream norm_key;
-        norm_key<< "ESO QC FLAT SED"<<ised+1<<" NORM ";
+        norm_key<< "ESO QC FLAT SED_"<<calib_slits[ised].slit_id()<<" NORM ";
         cpl_propertylist_append_float(sed_header, norm_key.str().c_str(),
                                       sed_norm[ised]);
     }
@@ -2112,6 +2112,8 @@ int vimos_calib_impl(cpl_frameset *frameset, cpl_parameterlist *parlist)
     cpl_image_delete(rainbow); rainbow = NULL;
 
     save_header = dfs_load_header(frameset, arc_tag, 0);
+    vimos_preoverscan ps_scan;
+    ps_scan.fix_wcs_trimm(save_header);
 
     cpl_image_turn(wavemap, rotate_back);
     if (dfs_save_image(frameset, wavemap, wavelength_map_tag, save_header,
@@ -2237,6 +2239,7 @@ std::auto_ptr<mosca::image> vimos_calib_flat_mos_create_master_flat
         //Only the structure is freed, the images are taken over by new_flat
         cpl_propertylist_delete(flat_header);
     }
+    cpl_image_delete(master_bias_var);
 
     if(!cpl_errorstate_is_equal(error_prevstate))
     {
