@@ -82,7 +82,8 @@ cpl_image_delete(skymap);                     \
 cpl_image_delete(smapped);                    \
 cpl_table_delete(offsets);                    \
 cpl_table_delete(sky);                        \
-cpl_image_delete(master_bias);                       \
+if(master_bias != NULL) cpl_image_delete(master_bias);              \
+if(master_bias_err != NULL) cpl_image_delete(master_bias_err);      \
 cpl_image_delete(spectra);                    \
 cpl_image_delete(coordinate);                 \
 cpl_image_delete(norm_flat);                  \
@@ -621,7 +622,7 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
             dfs_load_header(frameset, "MASTER_BIAS", 0);
     if (master_bias == NULL || master_bias_err == NULL || master_bias_header == NULL)
         vimos_science_exit("Cannot load master bias. Maybe error extension is missing");
-    cpl_image_delete(master_bias_err);
+    cpl_image_delete(master_bias_err); master_bias_err = NULL;
 
     /*
      * Loading input science
@@ -721,9 +722,9 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
         cpl_image_divide_scalar(spectra_err, exptime[0]);
         cpl_imagelist_set(all_science, spectra, 0);
         cpl_imagelist_set(all_science_err, spectra_err, 0);
-        cpl_image_delete(spectra_raw);
-        cpl_image_delete(science_var);
-        cpl_image_delete(science_notrim);
+        cpl_image_delete(spectra_raw); spectra_raw = NULL;
+        cpl_image_delete(science_var); science_var = NULL;
+        cpl_image_delete(science_notrim); science_notrim = NULL;
 
         for (i = 1; i < nscience; i++) {
 
@@ -837,8 +838,8 @@ int vimos_science_impl(cpl_frameset *frameset, cpl_parameterlist *parlist,
         cpl_image_delete(science_var);
         cpl_image_delete(science_notrim);
     }
-    cpl_image_delete(master_bias_var);
-    cpl_image_delete(master_bias);
+    cpl_image_delete(master_bias_var); master_bias_var = NULL;
+    cpl_image_delete(master_bias); master_bias = NULL;
 
     if (spectra == NULL)
         vimos_science_exit("Cannot load scientific frame");
