@@ -1322,7 +1322,7 @@ VimosBool writeFitsExtractionTable(VimosExtractionTable *extTable, fitsfile
 
   for (i = 0; i <= crvOrder; i++) {
     sprintf(colName,"CRV_POL_%d",i);
-    sprintf(ttype[10+i+1],colName);
+    sprintf(ttype[10+i+1], "%s", colName);
     tform[10+i+1] = "1D";
   }
   ttype[11+crvOrder+1]="CRVPOL_RMS";
@@ -1330,7 +1330,7 @@ VimosBool writeFitsExtractionTable(VimosExtractionTable *extTable, fitsfile
 
   for (i = 0; i <= invOrder; i++) {
     sprintf(colName,"INV_DIS_%d",i);
-    sprintf(ttype[11+crvOrder+1+i+1],colName); 
+    sprintf(ttype[11+crvOrder+1+i+1], "%s", colName); 
     tform[11+crvOrder+1+i+1] = "1D";
   }
   ttype[12+crvOrder+1+invOrder+1]="INVDIS_RMS";
