@@ -37,6 +37,7 @@
 #include <cpl_msg.h>
 
 #include "vmmatrix.h"
+#include "cpl.h"
 
 
 #define dtiny(a) ((a)<0?(a)> -1.e-30:(a)<1.e-30)
@@ -46,7 +47,7 @@ VimosFloatArray *newFloatArray(int len)
 {
   VimosFloatArray *tArray;
   
-  tArray = (VimosFloatArray *) pil_malloc(sizeof(VimosFloatArray));
+  tArray = (VimosFloatArray *) cpl_malloc(sizeof(VimosFloatArray));
 
   /* check if space was allocated */
   if (tArray == NULL) {
@@ -54,10 +55,10 @@ VimosFloatArray *newFloatArray(int len)
     return(NULL);
   }
   
-  tArray->data = (float *) pil_calloc(len, sizeof(float));
+  tArray->data = (float *) cpl_calloc(len, sizeof(float));
   if (tArray->data == NULL) {
     /* cleanup */
-    pil_free(tArray);
+    cpl_free(tArray);
     cpl_msg_error("newFloatArray","Allocation Error");
     return(NULL);
   }
@@ -73,8 +74,8 @@ void deleteFloatArray(VimosFloatArray *array)
   if (array == NULL) {
     return;
   }
-  pil_free(array->data);
-  pil_free(array);
+  cpl_free(array->data);
+  cpl_free(array);
 }
 
 
@@ -82,7 +83,7 @@ VimosIntArray *newIntArray(int len)
 {
   VimosIntArray *tArray;
   
-  tArray = (VimosIntArray *) pil_malloc(sizeof(VimosIntArray));
+  tArray = (VimosIntArray *) cpl_malloc(sizeof(VimosIntArray));
 
   /* check if space was allocated */
   if (tArray == NULL) {
@@ -90,10 +91,10 @@ VimosIntArray *newIntArray(int len)
     return(NULL);
   }
   
-  tArray->data = (int *) pil_calloc(len, sizeof(int));
+  tArray->data = (int *) cpl_calloc(len, sizeof(int));
   if (tArray->data == NULL) {
     /* cleanup */
-    pil_free(tArray);
+    cpl_free(tArray);
     cpl_msg_error("newIntArray","Allocation Error");
     return(NULL);
   }
@@ -109,8 +110,8 @@ void deleteIntArray(VimosIntArray *array)
   if (array == NULL) {
     return;
   }
-  pil_free(array->data);
-  pil_free(array);
+  cpl_free(array->data);
+  cpl_free(array);
 }
 
 
@@ -124,7 +125,7 @@ void deleteIntArray(VimosIntArray *array)
 VimosMatrix *newMatrix(int nr, int nc)
 {
   VimosMatrix *b;
-  b = (VimosMatrix *) pil_calloc(1,sizeof(VimosMatrix));
+  b = (VimosMatrix *) cpl_calloc(1,sizeof(VimosMatrix));
 
   /* check if space was allocated */
   if (b == NULL) {
@@ -132,10 +133,10 @@ VimosMatrix *newMatrix(int nr, int nc)
     return(NULL);
   }
 
-  b->data = (double *) pil_calloc(nr*nc, sizeof(double));
+  b->data = (double *) cpl_calloc(nr*nc, sizeof(double));
   if (b->data == NULL) {
     /* cleanup */
-    pil_free(b);
+    cpl_free(b);
     cpl_msg_error("newMatrix","Allocation Error");
     return(NULL);
   }
@@ -187,10 +188,10 @@ void deleteMatrix(VimosMatrix *a)
   }
   
   if (a->data != NULL) {
-    pil_free(a->data);
+    cpl_free(a->data);
   }
   
-  pil_free(a);
+  cpl_free(a);
   return;
 }
 
@@ -378,7 +379,7 @@ int gaussPivot(double *ptra, double *ptrc, int n)
   double max,r,t;
   double *ptrb;
   
-  ptrb = (double *)pil_calloc(n*n,sizeof(double));
+  ptrb = (double *)cpl_calloc(n*n,sizeof(double));
 
   /* check if space was allocated */
   if (ptrb == NULL) {
@@ -451,7 +452,7 @@ int gaussPivot(double *ptra, double *ptrc, int n)
     }
   }
   
-  pil_free(ptrb);
+  cpl_free(ptrb);
   return(1);
 
 #undef ABS
@@ -531,10 +532,10 @@ int *intVector(int nlow, int nhigh)
 {
   int *v;
  
-  v = (int *)pil_malloc((size_t) ((nhigh-nlow+1+RANGE_END)*sizeof(int)));
+  v = (int *)cpl_malloc((size_t) ((nhigh-nlow+1+RANGE_END)*sizeof(int)));
   if (!v)
     {
-     exit(-1);
+     abort();
     }
 
   return v-nlow+RANGE_END;
@@ -547,7 +548,7 @@ int *intVector(int nlow, int nhigh)
 void freeIntVector(int *v, int nlow, int nhigh)
 {
   nhigh = nhigh ;
-  pil_free((FREE_ARG) (v+nlow-RANGE_END));
+  cpl_free((FREE_ARG) (v+nlow-RANGE_END));
 }
 
 /* 
@@ -558,10 +559,10 @@ float *floatVector(int nlow, int nhigh)
 {
   float *v;
  
-  v = (float *)pil_malloc((size_t) ((nhigh-nlow+1+RANGE_END)*sizeof(float)));
+  v = (float *)cpl_malloc((size_t) ((nhigh-nlow+1+RANGE_END)*sizeof(float)));
   if (!v)
     {
-     exit(-1);
+     abort();
     }
   return v-nlow+RANGE_END;
 }
@@ -575,10 +576,10 @@ double *doubleVector(int nlow, int nhigh)
 {
   double *v;
  
-  v = (double *)pil_malloc((size_t) ((nhigh-nlow+1+RANGE_END)*sizeof(double)));
+  v = (double *)cpl_malloc((size_t) ((nhigh-nlow+1+RANGE_END)*sizeof(double)));
   if (!v)
     {
-     exit(-1);
+     abort();
     }
   return v-nlow+RANGE_END;
 }
@@ -590,7 +591,7 @@ double *doubleVector(int nlow, int nhigh)
 void freeFloatVector(float *v, int nlow, int nhigh)
 {
   nhigh = nhigh ;
-  pil_free((FREE_ARG) (v+nlow-RANGE_END));
+  cpl_free((FREE_ARG) (v+nlow-RANGE_END));
 }
 
 /*
@@ -600,7 +601,7 @@ void freeFloatVector(float *v, int nlow, int nhigh)
 void freeDoubleVector(double *v, int nlow, int nhigh)
 {
   nhigh = nhigh ;
-  pil_free((FREE_ARG) (v+nlow-RANGE_END));
+  cpl_free((FREE_ARG) (v+nlow-RANGE_END));
 }
 
 /*
@@ -613,20 +614,20 @@ float **Matrix(int nrl, int nrh, int ncl, int nch)
   float **m;
  
   /* allocate pointers to rows */
-  m = (float **) pil_malloc((size_t)((nrow+RANGE_END)*sizeof(float*)));
+  m = (float **) cpl_malloc((size_t)((nrow+RANGE_END)*sizeof(float*)));
         
   if (!m)
     {
-     exit(-1);
+     abort();
     }
   m += RANGE_END;
   m -= nrl;
   /* allocate rows and set pointers to them */
-  m[nrl] = (float *) pil_malloc((size_t)((nrow*ncol+RANGE_END)*sizeof(float)));
+  m[nrl] = (float *) cpl_malloc((size_t)((nrow*ncol+RANGE_END)*sizeof(float)));
 
   if (!m[nrl])
     {
-     exit(-1);
+     abort();
     }
 
   m[nrl] += RANGE_END;
@@ -646,8 +647,8 @@ void freeMatrix(float **m, int nrl, int nrh, int ncl, int nch)
 {
   nrh = nrh;
   nch = nch;
-  pil_free((FREE_ARG) (m[nrl]+ncl-RANGE_END));
-  pil_free((FREE_ARG) (m+nrl-RANGE_END));
+  cpl_free((FREE_ARG) (m[nrl]+ncl-RANGE_END));
+  cpl_free((FREE_ARG) (m+nrl-RANGE_END));
 }
 
 double **doubleMatrix(long nrl, long nrh, long ncl, long nch)
@@ -657,7 +658,7 @@ double **doubleMatrix(long nrl, long nrh, long ncl, long nch)
 	double **m;
 
 	/* allocate pointers to rows */
-	m=(double **) pil_malloc((size_t)((nrow+RANGE_END)*sizeof(double*)));
+	m=(double **) cpl_malloc((size_t)((nrow+RANGE_END)*sizeof(double*)));
 	if (!m) {
 	  exit(-1) ;
 	}
@@ -665,7 +666,7 @@ double **doubleMatrix(long nrl, long nrh, long ncl, long nch)
 	m -= nrl;
 
 	/* allocate rows and set pointers to them */
-	m[nrl]=(double *) pil_malloc((size_t)((nrow*ncol+RANGE_END)*sizeof(double)));
+	m[nrl]=(double *) cpl_malloc((size_t)((nrow*ncol+RANGE_END)*sizeof(double)));
 	if (!m[nrl]) exit(-1);
 	m[nrl] += RANGE_END;
 	m[nrl] -= ncl;
@@ -682,8 +683,8 @@ void freeDoubleMatrix(double **m, long nrl, long nrh, long ncl, long nch)
 {
   nrh = nrh;
   nch = nch;
-  pil_free((FREE_ARG) (m[nrl]+ncl-RANGE_END));
-  pil_free((FREE_ARG) (m+nrl-RANGE_END));
+  cpl_free((FREE_ARG) (m[nrl]+ncl-RANGE_END));
+  cpl_free((FREE_ARG) (m+nrl-RANGE_END));
 }
 
 
@@ -700,11 +701,11 @@ float **convertMatrix(float *a, int nrl, int nrh, int ncl, int nch)
   float **m;
 
   /* allocate pointers to rows */
-  m = (float **) pil_malloc((size_t) ((nrow+RANGE_END)*sizeof(float*)));
+  m = (float **) cpl_malloc((size_t) ((nrow+RANGE_END)*sizeof(float*)));
 
   if (!m)
     {
-     exit(-1);
+     abort();
     }
 
   m += RANGE_END;
@@ -725,7 +726,7 @@ void freeConvertedMatrix(float **b, int nrl, int nrh, int ncl, int nch)
   nrh = nrh ;
   ncl = ncl ;
   nch = nch ;
-  pil_free((FREE_ARG) (b+nrl-RANGE_END));
+  cpl_free((FREE_ARG) (b+nrl-RANGE_END));
 }
 
 /* commented out to avoid warning message sin compilation
@@ -758,15 +759,15 @@ VimosFloat2DArray *newFloat2DArray(int len)
 {
   VimosFloat2DArray *tArray;
   
-  tArray = (VimosFloat2DArray *) pil_malloc(sizeof(VimosFloat2DArray));
+  tArray = (VimosFloat2DArray *) cpl_malloc(sizeof(VimosFloat2DArray));
   if (tArray == NULL) {
     exit(-2);
   }
   
-  tArray->x = (float *) pil_calloc(len, sizeof(float));
-  tArray->y = (float *) pil_calloc(len, sizeof(float));
+  tArray->x = (float *) cpl_calloc(len, sizeof(float));
+  tArray->y = (float *) cpl_calloc(len, sizeof(float));
   if ( (tArray->x == NULL) || (tArray->y == NULL) ) {
-    pil_free(tArray);
+    cpl_free(tArray);
     exit(-1);
   }
   
@@ -785,9 +786,9 @@ void deleteFloat2DArray(VimosFloat2DArray *array)
   if (array == NULL) {
     return;
   }
-  pil_free(array->x);
-  pil_free(array->y);
-  pil_free(array);
+  cpl_free(array->x);
+  cpl_free(array->y);
+  cpl_free(array);
 }
 
 /*
