@@ -86,7 +86,7 @@
  * assuming that the buffer's size is @b n + 1!
  */
 
-inline const char *
+const char *
 sextGetFileName(char *buffer, const char *path, size_t n)
 {
 
@@ -441,7 +441,7 @@ int sextSaveParameters(FILE *pfile, SextParameter *columns)
  * absolute path is returned.
  */
 
-inline const char *sextGetStarNnwName(void)
+const char *sextGetStarNnwName(void)
 {
 
   const char *name = pilDfsDbGetString(SEXTRACTOR_GROUP, "StarNnwName");
@@ -467,7 +467,7 @@ inline const char *sextGetStarNnwName(void)
  * absolute path is returned.
  */
 
-inline const char *sextGetFilterName(void)
+const char *sextGetFilterName(void)
 {
 
   const char *name = pilDfsDbGetString(SEXTRACTOR_GROUP, "FilterName");
@@ -493,7 +493,7 @@ inline const char *sextGetFilterName(void)
  * path is returned.
  */
 
-inline const char *sextGetAssocName(void)
+const char *sextGetAssocName(void)
 {
 
   const char *name = pilDfsDbGetString(SEXTRACTOR_GROUP, "AssocName");
@@ -519,7 +519,7 @@ inline const char *sextGetAssocName(void)
  * absolute path is returned.
  */
 
-inline const char *sextGetCheckImageName(void)
+const char *sextGetCheckImageName(void)
 {
 
   const char *name = pilDfsDbGetString(SEXTRACTOR_GROUP, "CheckImageName");
@@ -545,7 +545,7 @@ inline const char *sextGetCheckImageName(void)
  * is returned.
  */
 
-inline const char *sextGetFlagImageName(void)
+const char *sextGetFlagImageName(void)
 {
 
   const char *name = pilDfsDbGetString(SEXTRACTOR_GROUP, "FlagImage");
@@ -571,7 +571,7 @@ inline const char *sextGetFlagImageName(void)
  * absolute path is returned.
  */
 
-inline const char *sextGetWeightImageName(void)
+const char *sextGetWeightImageName(void)
 {
 
   const char *name = pilDfsDbGetString(SEXTRACTOR_GROUP, "WeightImage");
@@ -597,7 +597,7 @@ inline const char *sextGetWeightImageName(void)
  * absolute path to the SExtractor executable, is returned.
  */
 
-inline const char *sextGetSextractorPath(void)
+const char *sextGetSextractorPath(void)
 {
 
   static char sextractor[PATHNAME_MAX + 1] = "";
