@@ -52,6 +52,7 @@
 #include "vmimage.h"
 #include "vmtable.h"
 #include "vmutils.h"
+#include "cpl.h"
 
 
 /* Set parameters for PAF writer */
@@ -370,13 +371,13 @@ applyListSelection(VimosImage **imageList, float *imageProperty,
    * property value.
    */
 
-  rejectedList = (VimosImage **)pil_malloc(imageCount * sizeof(VimosImage *));
+  rejectedList = (VimosImage **)cpl_malloc(imageCount * sizeof(VimosImage *));
   if (!rejectedList)
     return -1;
 
-  rejectedProperty = (float *)pil_malloc(imageCount * sizeof(float));
+  rejectedProperty = (float *)cpl_malloc(imageCount * sizeof(float));
   if (!rejectedProperty) {
-    pil_free(rejectedList);
+    cpl_free(rejectedList);
     return -1;
   }
 
@@ -421,8 +422,8 @@ applyListSelection(VimosImage **imageList, float *imageProperty,
    * Cleanup
    */
 
-  pil_free(rejectedList);
-  pil_free(rejectedProperty);
+  cpl_free(rejectedList);
+  cpl_free(rejectedProperty);
 
   return j;
 
@@ -497,13 +498,13 @@ remapFloatsLikeImages(VimosImage **original, VimosImage **sorted,
     }
   }
 
-  if (!(holder = (float *)pil_malloc(imageCount*sizeof(float)))) {
+  if (!(holder = (float *)cpl_malloc(imageCount*sizeof(float)))) {
     cpl_msg_debug(modName, "Problems with memory allocation");
     return EXIT_FAILURE;
   }
 
-  if (!(error = (int *)pil_malloc(imageCount*sizeof(int)))) {
-    pil_free(holder);
+  if (!(error = (int *)cpl_malloc(imageCount*sizeof(int)))) {
+    cpl_free(holder);
     cpl_msg_debug(modName, "Problems with memory allocation");
     return EXIT_FAILURE;
   }
@@ -523,8 +524,8 @@ remapFloatsLikeImages(VimosImage **original, VimosImage **sorted,
 
   for (i = 0; i < imageCount;  i++) {   /* Error checking */
     if (error[i]) {
-      pil_free(holder);
-      pil_free(error);
+      cpl_free(holder);
+      cpl_free(error);
       cpl_msg_debug(modName, "Input image arrays are not comparable");
       return EXIT_FAILURE;
     }
@@ -533,8 +534,8 @@ remapFloatsLikeImages(VimosImage **original, VimosImage **sorted,
   for (i = 0; i < imageCount;  i++)  /* New sorting overwrites old one */
     array[i] = holder[i];
 
-  pil_free(holder);
-  pil_free(error);
+  cpl_free(holder);
+  cpl_free(error);
   
   return EXIT_SUCCESS;
 
@@ -609,13 +610,13 @@ remapDoublesLikeImages(VimosImage **original, VimosImage **sorted,
     }
   }
 
-  if (!(holder = (double *)pil_malloc(imageCount*sizeof(double)))) {
+  if (!(holder = (double *)cpl_malloc(imageCount*sizeof(double)))) {
     cpl_msg_debug(modName, "Problems with memory allocation");
     return EXIT_FAILURE;
   }
 
-  if (!(error = (int *)pil_malloc(imageCount*sizeof(int)))) {
-    pil_free(holder);
+  if (!(error = (int *)cpl_malloc(imageCount*sizeof(int)))) {
+    cpl_free(holder);
     cpl_msg_debug(modName, "Problems with memory allocation");
     return EXIT_FAILURE;
   }
@@ -635,8 +636,8 @@ remapDoublesLikeImages(VimosImage **original, VimosImage **sorted,
 
   for (i = 0; i < imageCount;  i++) {   /* Error checking */
     if (error[i]) {
-      pil_free(holder);
-      pil_free(error);
+      cpl_free(holder);
+      cpl_free(error);
       cpl_msg_debug(modName, "Input image arrays are not comparable");
       return EXIT_FAILURE;
     }
@@ -645,8 +646,8 @@ remapDoublesLikeImages(VimosImage **original, VimosImage **sorted,
   for (i = 0; i < imageCount;  i++)  /* New sorting overwrites old one */
     array[i] = holder[i];
 
-  pil_free(holder);
-  pil_free(error);
+  cpl_free(holder);
+  cpl_free(error);
   
   return EXIT_SUCCESS;
 
@@ -662,8 +663,8 @@ void sortN (int ncol, float **ra, int sortCol, int fromRow, int forRows)
         int i,j,i1,*iwksp;
         float *wksp;
 
-        iwksp=pil_calloc(forRows, sizeof(int *));
-        wksp=pil_calloc(forRows, sizeof(float *));
+        iwksp=cpl_calloc(forRows, sizeof(int *));
+        wksp=cpl_calloc(forRows, sizeof(float *));
         j=0;
         for (i=fromRow;i<fromRow+forRows;i++){
           wksp[j]=ra[i][sortCol];
@@ -682,8 +683,8 @@ void sortN (int ncol, float **ra, int sortCol, int fromRow, int forRows)
             i1++;
           }
         }
-        pil_free(wksp);
-        pil_free(iwksp);
+        cpl_free(wksp);
+        cpl_free(iwksp);
 }
 
 
