@@ -41,6 +41,7 @@
 #include "vmmath.h"
 #include "vmimgutils.h"
 #include "vmwcsutils.h"
+#include "cpl.h"
 
 
 /*-----------------------------------------------------------------------------
@@ -184,10 +185,10 @@ VimosBool vimosFitMatch(struct WorldCoor *wcs,VimosTable *o_star, int nmatch)
 
   /* Allocate memory for working arrays */
   
-   if(!(in_ximage = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(in_yimage = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(in_xworld = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(in_yworld = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ){
+   if(!(in_ximage = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(in_yimage = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(in_xworld = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(in_yworld = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ){
      cpl_msg_error(modName, "Could not alloc memory for finding plate solution");
      return VM_FALSE;
    }
@@ -235,9 +236,9 @@ VimosBool vimosFitMatch(struct WorldCoor *wcs,VimosTable *o_star, int nmatch)
    setnfit(-125);
    nfit = 3;
 
-   resid = (double *)pil_malloc(nmatch * sizeof(double));
-   xe = (double *)pil_malloc(nmatch * sizeof(double));
-   ye = (double *)pil_malloc(nmatch * sizeof(double));
+   resid = (double *)cpl_malloc(nmatch * sizeof(double));
+   xe = (double *)cpl_malloc(nmatch * sizeof(double));
+   ye = (double *)cpl_malloc(nmatch * sizeof(double));
    dmatch = (double)nmatch;
    dmatch1 = (double)(nmatch - 1);
 
@@ -332,10 +333,10 @@ VimosBool vimosFitMatch(struct WorldCoor *wcs,VimosTable *o_star, int nmatch)
 
    /* Free memory */
    
-   if(in_ximage) pil_free(in_ximage);
-   if(in_yimage) pil_free(in_yimage);
-   if(in_xworld) pil_free(in_xworld);
-   if(in_yworld) pil_free(in_yworld);
+   if(in_ximage) cpl_free(in_ximage);
+   if(in_yimage) cpl_free(in_yimage);
+   if(in_xworld) cpl_free(in_xworld);
+   if(in_yworld) cpl_free(in_yworld);
    
    return VM_TRUE;
 }
@@ -409,14 +410,14 @@ VimosBool vimosFitPlate(struct WorldCoor * wcs,
    
  
    
-   if(!(x_1 = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(y_1 = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(sig = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(x_2 = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(y_2 = (double *)pil_calloc((size_t)nmatch , sizeof (double))) ||
-      !(ifit = (int *)pil_calloc ((size_t)VMMAXPAR , sizeof(int))) ||
-      !(xfit_coeff = (double *)pil_calloc((size_t)VMMAXPAR , sizeof(double))) ||
-      !(yfit_coeff = (double *)pil_calloc((size_t)VMMAXPAR , sizeof(double))) ){
+   if(!(x_1 = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(y_1 = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(sig = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(x_2 = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(y_2 = (double *)cpl_calloc((size_t)nmatch , sizeof (double))) ||
+      !(ifit = (int *)cpl_calloc ((size_t)VMMAXPAR , sizeof(int))) ||
+      !(xfit_coeff = (double *)cpl_calloc((size_t)VMMAXPAR , sizeof(double))) ||
+      !(yfit_coeff = (double *)cpl_calloc((size_t)VMMAXPAR , sizeof(double))) ){
       cpl_msg_error(modName,"Could not alloc memory for finding plate solution");
       return VM_FALSE;
    }
@@ -476,14 +477,14 @@ VimosBool vimosFitPlate(struct WorldCoor * wcs,
  
    /* Free memory */
    
-   if(x_1) pil_free(x_1);
-   if(y_1) pil_free(y_1);
-   if(sig) pil_free(sig);
-   if(x_2) pil_free(x_2);
-   if(y_2) pil_free(y_2);
-   if(ifit) pil_free(ifit);
-   if(xfit_coeff) pil_free(xfit_coeff);
-   if(yfit_coeff) pil_free(yfit_coeff);
+   if(x_1) cpl_free(x_1);
+   if(y_1) cpl_free(y_1);
+   if(sig) cpl_free(sig);
+   if(x_2) cpl_free(x_2);
+   if(y_2) cpl_free(y_2);
+   if(ifit) cpl_free(ifit);
+   if(xfit_coeff) cpl_free(xfit_coeff);
+   if(yfit_coeff) cpl_free(yfit_coeff);
    
    return VM_TRUE;
 }
@@ -678,7 +679,7 @@ struct WorldCoor *rdimage(VimosDescriptor *descs)
    }
    
 
-   wcs = (struct WorldCoor *)pil_calloc(1, sizeof(struct WorldCoor));
+   wcs = (struct WorldCoor *)cpl_calloc(1, sizeof(struct WorldCoor));
 
    /* Set WCSLIB flags so that structures will be reinitialized */
    wcs->cel.flag = 0;
@@ -721,25 +722,25 @@ struct WorldCoor *rdimage(VimosDescriptor *descs)
    wcs->nypix = (double) nypix;
 
    descName = (char *)pilKeyTranslate("Ctype",1);
-   ctype1 = (char *)pil_malloc(9*sizeof(char)); //ctype[4][9] in vimoswcs.h
+   ctype1 = (char *)cpl_malloc(9*sizeof(char)); //ctype[4][9] in vimoswcs.h
    if(!(readStringDescriptor(descs, descName,
                              ctype1, comment))) {
     cpl_msg_error(modName,"Descriptor %s not found", descName);
-    pil_free(descName);
+    cpl_free(descName);
 
     return NULL;
    }
-   pil_free(descName);
+   cpl_free(descName);
    descName = (char *)pilKeyTranslate("Ctype",2);
-   ctype2 = (char *)pil_malloc(9*sizeof(char)); //ctype[4][9] in vimoswcs.h
+   ctype2 = (char *)cpl_malloc(9*sizeof(char)); //ctype[4][9] in vimoswcs.h
    if(!(readStringDescriptor(descs, descName,
                              ctype2, comment))) {
     cpl_msg_error(modName,"Descriptor %s not found", descName);
-    pil_free(descName);
+    cpl_free(descName);
 
     return NULL;
    }
-   pil_free(descName);
+   cpl_free(descName);
 
 
    strcpy (wcs->ctype[0], ctype1);
@@ -963,14 +964,14 @@ computeVirtualPixels(VimosDescriptor *descs, VimosTable *ostar,
       return VM_FALSE;
   } 
 
-  x_coeff = (double *)pil_calloc(ipow(nx + 1, 2), sizeof(double));
-  y_coeff = (double *)pil_calloc(ipow(ny + 1, 2), sizeof(double));
+  x_coeff = (double *)cpl_calloc(ipow(nx + 1, 2), sizeof(double));
+  y_coeff = (double *)cpl_calloc(ipow(ny + 1, 2), sizeof(double));
 
   if (getCcdSky(descs, x_coeff, y_coeff) == VM_FALSE) {
       cpl_msg_error(id, "Cannot read CCD to sky transformation matrix!");
 
-      pil_free(x_coeff);
-      pil_free(y_coeff);
+      cpl_free(x_coeff);
+      cpl_free(y_coeff);
 
       return VM_FALSE;
   }
@@ -988,8 +989,8 @@ computeVirtualPixels(VimosDescriptor *descs, VimosTable *ostar,
   if (readIntDescriptor(descs, key, &xCenter, NULL) == VM_FALSE) {
       cpl_msg_error(id, "Missing keyword `%s'", key);
 
-      pil_free(x_coeff);
-      pil_free(y_coeff);
+      cpl_free(x_coeff);
+      cpl_free(y_coeff);
 
       return VM_FALSE;
   }
@@ -998,8 +999,8 @@ computeVirtualPixels(VimosDescriptor *descs, VimosTable *ostar,
   if (readIntDescriptor(descs, key, &yCenter, NULL) == VM_FALSE) {
       cpl_msg_error(id, "Missing keyword `%s'", key);
 
-      pil_free(x_coeff);
-      pil_free(y_coeff);
+      cpl_free(x_coeff);
+      cpl_free(y_coeff);
 
       return VM_FALSE;
   }
@@ -1039,8 +1040,8 @@ computeVirtualPixels(VimosDescriptor *descs, VimosTable *ostar,
   }
 
 
-  pil_free(x_coeff);
-  pil_free(y_coeff);
+  cpl_free(x_coeff);
+  cpl_free(y_coeff);
 
   return VM_TRUE;
 
